<?php
class Netzkollektiv_Lexoffice_Model_Sync {
    protected $_documents = null;
    protected $_pdfRenderer = null;
    protected $_api;

    protected $_skipTotals = array('subtotal','grand_total','tax_amount','discount_amount');

    public function __construct() {
        $this->_api = Mage::helper('lexoffice')->getApi();
    }

    public function sync() {
        $errors = array();
        foreach ($this->_documents as $document) {
            try {

                $this->_sync($document);
                
                $document->setLexofficeSent(true)->save();

            } catch (\Lexoffice\Lexception $e) {
                throw new \Lexoffice\Lexception($e);
            } catch (Exception $e) {
                $errors[] = '['.$document->getIncrementId().'] '.$e->getMessage();
            }
        }
        if (count($errors) > 0) {
            throw new Exception(implode("\n",$errors));
        }
    }

    protected function _getDocumentType($document) {
        if ($document instanceof Mage_Sales_Model_Order_Invoice) {
            return 'salesinvoice';
        }
        if ($document instanceof Mage_Sales_Model_Order_Creditmemo) {
            return 'salescreditnote';
        }
    }

    protected function _getPdfFile($document) {
        $pdfString = $this->_pdfRenderer->getPdf(array($document))->render();
        $filename = tempnam('/tmp','lexoffice').'.pdf';
        file_put_contents($filename, $pdfString);
        return $filename;
    }

    public function _sync($document) {

        $useCollectiveContact = Mage::getStoreConfig('lexoffice/customers/use_collective_contact');
        $contactId = (boolean)$useCollectiveContact;
        if (!$contactId) {
            if ($document->getOrder()->getCustomerIsGuest()) {
                $contactId = $this->_addGuest($document->getOrder()->getBillingAddress());
            } else {
                $customer = Mage::getModel('customer/customer')->load($document->getOrder()->getCustomerId());
                $contactId = $this->_addCustomer($customer);
            }
        }

        $voucherId = $this->_addVoucher($contactId, $document);
        $this->_addVoucherFile($voucherId, $this->_getPdfFile($document));
    }

    public function setDocuments($documents) {
        $this->_documents = $documents;
        return $this;
    }

    public function setPdfRenderer($renderer) {
        $this->_pdfRenderer = $renderer;
        return $this;
    }

    public function _addGuest($address) {
        return $this->_api->addContact(json_encode(array(
            'version' => 0,
            'roles' => array('customer'=> (object)array()),
            'person' => array(
                'salutation' => 'Herr',
                'firstName' => $address->getFirstname(),
                'lastName' => $address->getLastname()
            ),
            'note' => ''
        )))->id;
    }

    public function _addCustomer($customer) {
        if ($contactId = $customer->getLexofficeContactId()) {
            return $contactId;
        }

        $data = array(
            'version' => 0,
            'roles' => array('customer'=> (object)array($customer->getId())),
            'addresses' => array(
                'shipping' => $this->_getAddressFragment($customer->getPrimaryShippingAddress()),
                'billing'  => $this->_getAddressFragment($customer->getPrimaryBillingAddress())
            ),
            'note' => ''
        );

        $address = $customer->getPrimaryBillingAddress();
        if ($address->getCompany()) {
            $data = array_merge($data,array(
                'company' => array(
                    'name' => $address->getCompany(),
                    'vatRegistrationId' => $customer->getTaxVat(),
                    'contactPersons' => array(
                        $this->_getPersonFragment($customer, true)
                    )
                )
            ));
        } else {
           $data = array_merge($data,array(
               'person' => $this->_getPersonFragment($customer, false),
            )); 
        }

        $contactId = $this->_api->addContact(json_encode($data))->id;

        $customer->setLexofficeContactId($contactId)
            ->save();

        return $contactId;
    }

    protected function _getPersonFragment($customer, $includeCommunication = true) {
        $fragment = array(
            'salutation' => 'Herr',
            'firstName' => $customer->getFirstname(),
            'lastName' => $customer->getLastname(),
            'emailAddress' => $customer->getEmail(), 
            'phoneNumber' => $customer->getPrimaryBillingAddress()->getTelephone()
        );
        if (!$includeCommunication) {
            unset($framgent['emailAddress'],$fragment['phoneNumber']);
        }
        return $fragment;
    }

    protected function _getAddressFragment($address) {
        return array(array(
            'street' => implode(' ',$address->getStreet()),
            'zip' => $address->getPostcode(),
            'city'=> $address->getCity(),
            'countryCode' => $address->getCountryId()
        ));
    }

    protected function _addVoucher($contactId, $document) {

        $voucher = array(
           'type' => $this->_getDocumentType($document),
           'voucherNumber' => $document->getIncrementId(),
           'voucherDate' => date('Y-m-d',strtotime($document->getCreatedAt())),
           'dueDate' => date('Y-m-d',strtotime($document->getCreatedAt())),
           'totalGrossAmount' => round($document->getBaseGrandTotal(),2),
           'totalTaxAmount' => round($document->getBaseTaxAmount(),2),
           'taxType' => 'gross',
           'useCollectiveContact' => ($contactId === true),
           'remark' => '',
           'contactId'=> $contactId,
           'voucherItems' => array(),
           'voucherStatus' => 'open'
        );
        if ($voucher['useCollectiveContact'] === true) {
            unset($voucher['contactId']);
        }

        $voucher['voucherItems'] = $this->_getGroupedItems(
            $this->_getApplicableItems($document)
        );

        return $this->_api
            ->addVoucher(json_encode($voucher))
            ->id;
    }

    protected function _getApplicableItems($document) {
        $items = array();
        foreach ($document->getAllItems() as $_item) {
            if ($_item->getOrderItem()->getParentItem()) {
                continue;
            }

            $items[] = array(
                'grossAmount' => $_item->getData('base_row_total_incl_tax') - $_item->getData('base_discount_amount'),
                'taxAmount' => $_item->getBaseTaxAmount()
            );
        }

        foreach (Mage::helper('lexoffice/totals')->getTotals($document) as $total) {
            if (in_array($total->getCode(),$this->_skipTotals)) {
                continue;
            }
            $items[] = array(
                'grossAmount' => $total->getAmountInclTax(),
                'taxAmount' => $total->getAmountInclTax() - $total->getAmount(),
            );
        }
        return $items;
    }

    protected function _getGroupedItems($items) {
        $voucherItems = array();
        foreach ($items as $item) {
            $taxPercent = round(($item['grossAmount'] / ($item['grossAmount'] - $item['taxAmount']) - 1) * 100);

            if (!isset($voucherItems[$taxPercent])) {
                $voucherItems[$taxPercent] = array(
                    'amount' => 0,
                    'taxAmount' => 0,
                    'taxRatePercent' => $taxPercent,
                    'categoryId' => Mage::getStoreConfig('lexoffice/accounts/revenue')
                );
            }

            $voucherItems[$taxPercent]['amount'] += $item['grossAmount'];
            $voucherItems[$taxPercent]['taxAmount'] += $item['taxAmount'];
        }

        foreach ($voucherItems as $taxPercent => $item) {
            $voucherItems[$taxPercent]['amount'] = round($item['amount'],2);
            $voucherItems[$taxPercent]['taxAmount'] = round($item['taxAmount'],2);
        }
        return array_values($voucherItems);
    }

    public function _addVoucherFile($voucherId, $filename) {
        return $this->_api
            ->addVoucherFile($voucherId, $filename)
            ->id;
    }
}
