<?php
namespace Lexoffice;

class Api {
    protected $_baseUri = 'https://lexoffice.netzkollektiv.com/';
    protected $_token = null;

    public function __construct($baseUrl, $token) {
        $this->_baseUrl = $baseUrl;
        $this->setToken($token);
    }

    public function _getClient() {
        $client = new \Zend_Http_Client();
        $client->setHeaders('X-Subscription-Token', json_encode(array(
            'id' => isset($this->_token->subscription_id) ? $this->_token->subscription_id : '',
            'url' => $this->_baseUrl
        )));
        return $client;
    }

    public function getAuthUrl($redirectUri) {
        $uri = $this->_baseUri.'_internal/auth?redirect_uri='.rawurlencode($redirectUri);
        if ($this->_token->subscription_id !== null) {
            $uri.= '&subscription_id='.rawurlencode($this->_token->subscription_id);
        }
        return $uri;
    }

    public function getAccessToken($authCode) {

        $client = $this->_getClient();
        $client->setUri($this->_baseUri.'api/oauth2/token')
            ->setMethod('POST')
            ->setParameterGet(array(
                'grant_type' => 'authorization_code',
                'code' => $authCode
            ));
        $result = $this->_handleSend($client);

        if (!isset($result->access_token)) {
            throw new \Exception('could not get access token, please check auth code');
        }
        $this->setToken($result);
        return $result;
    }

    protected function _refreshToken() {

        $client = $this->_getClient();
        $client->setUri($this->_baseUri.'api/oauth2/token')
            ->setMethod('POST')
            ->setParameterGet(array(
                'grant_type' => 'refresh_token',
                'refresh_token' => $this->_token->refresh_token
            ));
        $result = $this->_handleSend($client);

        if (!isset($result->access_token)) {
            throw new \Exception('could not refresh access token');
        }
        $this->setToken($result);
    }

    public function setToken($token) {
        // do not loose previously set token
        if (isset($this->_token->subscription_id) && !isset($token->subscription_id)) {
            $token->subscription_id = $this->_token->subscription_id;
        }
        $this->_token = $token;
        return $this;
    }

    protected function _handleSend($request) {
        $result = $request->request();
        $body = json_decode($result->getBody());
        if ($result->isSuccessful()) {
            return $body;
        }
        if (isset($body->error)) {
            if ($body->error == 'invalid_token') {
                try {
                    $this->_refreshToken();
                    $request->setHeaders('Authorization','Bearer '.$this->_token->access_token);
                    return $this->_handleSend($request);
                } catch (Exception $e) {
                    throw new TokenException(); 
                }
            } elseif ($body->error == 'invalid_grant') {
                throw new TokenException(); 
            } elseif ($body->error == 'expired') {
                throw new LicenseException();
            }
            throw new \Exception($body->error_description);
        }
        if (isset($body->IssueList) && is_array($body->IssueList)) {
            $issues = array();
            foreach ($body->IssueList as $issue) {
                $issues[] = $issue->type.': '.$issue->source.': '.$issue->i18nKey;
            }
            throw new \Exception(implode(';',$issues));
        }
        throw new \Exception('unknown error'.PHP_EOL
            .(string)$request->getLastRequest().PHP_EOL
            .(string)$request->getLastResponse().PHP_EOL
        );
    }

    public function addContact($contact) {
        $client = $this->_getClient();
        $client->setUri($this->_baseUri.'partner-api/v1/contacts')
            ->setMethod('POST')
            ->setHeaders(array(
                'Accept' => 'application/json',
                'Authorization' => 'Bearer '.$this->_token->access_token,
                'Content-Type' => 'application/json'
            ))
            ->setRawData($contact);
        return $this->_handleSend($client);
    }

    public function addVoucher($voucher) {
        $client = $this->_getClient();
        $client->setUri($this->_baseUri.'partner-api/v1/vouchers')
            ->setMethod('POST')
            ->setHeaders(array(
                'Accept' => 'application/json',
                'Authorization' => 'Bearer '.$this->_token->access_token,
                'Content-Type' => 'application/json'
            ))->setRawData($voucher);
        return $this->_handleSend($client);
    }

    public function addVoucherFile($voucherId, $filename) {
        $client = $this->_getClient();
        $client->setUri($this->_baseUri.'partner-api/v1/vouchers/'.$voucherId.'/files')
            ->setMethod('POST')
            ->setHeaders(array(
                'Authorization' => 'Bearer '.$this->_token->access_token,
            ))
            ->setFileUpload($filename,'file')
            ->setParameterPost(array('type'=>'voucher'));
         return $this->_handleSend($client);
    }

    public function getProfile() {
        $client = $this->_getClient();
        $client->setUri($this->_baseUri.'partner-api/v1/profile')
            ->setMethod('GET')
            ->setHeaders(array(
                'Accept' => 'application/json',
                'Authorization' => 'Bearer '.$this->_token->access_token,
            ));
        return $this->_handleSend($client);
    }

    /* Subscription Handling */
    public function getSubscription() {
        $client = $this->_getClient();
        $client->setUri($this->_baseUri.'_internal/subscription/'.$this->_token->subscription_id.'/info')
            ->setMethod('GET')
            ->setHeaders(array(
                'Accept' => 'application/json',
            ));
        return $this->_handleSend($client);
    }

    public function cancelSubscription() {
        $client = $this->_getClient();
        $client->setUri($this->_baseUri.'_internal/subscription/'.$this->_token->subscription_id.'/cancel')
            ->setMethod('GET')
            ->setHeaders(array(
                'Accept' => 'application/json',
            ));
        return $this->_handleSend($client);
    }
}
